﻿using System;
using System.Diagnostics;

namespace BbqpCmcs.AtomicOperators
{
	class FlipX : AtomicOperator
	{
		public FlipX()
		{
			Name = "FlpX";
		}

		public override void Run(InternalBbqpSolution solution, Random random)
		{
			for (int i = solution.Instance.M - 1; i >= 0; i--)
			{
				long newObjective = solution.CSum;

				if (solution.GetX(i))
				{
					newObjective -= solution.Instance.GetC(i);
						
					for (int j = solution.Instance.N - 1; j >= 0; j--)
					{
						var colSum = solution.GetColSum(j);
						colSum -= solution.Instance[i, j];
						if (colSum > 0)
							newObjective += colSum;
					}
				}
				else
				{
					newObjective += solution.Instance.GetC(i);

					for (int j = solution.Instance.N - 1; j >= 0; j--)
					{
						var colSum = solution.GetColSum(j);
						colSum += solution.Instance[i, j];
						if (colSum > 0)
							newObjective += colSum;
					}
				}

				if (newObjective > solution.Objective)
				{
					solution.SetX(i, !solution.GetX(i));
					for (int j = solution.Instance.N - 1; j >= 0; j--)
					{
						var colSum = solution.GetColSum(j);
						if (colSum > 0)
							solution.SetY(j, true);
						else if (colSum < 0)
							solution.SetY(j, false);
					}

					Debug.Assert(newObjective == solution.Objective);
				}
			}
		}

		public override bool FindsLocalMinimum
		{
			get { return false; }
		}

		public override bool CanWorsen
		{
			get { return false; }
		}

		public override bool IsDeterministicImprovement
		{
			get { return true; }
		}

		public override bool CanChangeSolution(AtomicOperator prevComponent, bool prevImprovedSolution)
		{
			if (prevComponent is FlipX && !prevImprovedSolution)
				return false;

			return true;
		}
	}
}
