﻿using System;
using System.Diagnostics;

namespace BbqpCmcs.AtomicOperators
{
	class FlipY : AtomicOperator
	{
		public FlipY()
		{
			Name = "FlpY";
		}

		public override void Run(InternalBbqpSolution solution, Random random)
		{
			for (int j = solution.Instance.N - 1; j >= 0; j--)
			{
				long newObjective = solution.DSum;

				if (solution.GetY(j))
				{
					newObjective -= solution.Instance.GetD(j);

					for (int i = solution.Instance.M - 1; i >= 0; i--)
					{
						var rowSum = solution.GetRowSum(i);
						rowSum -= solution.Instance[i, j];
						if (rowSum > 0)
							newObjective += rowSum;
					}
				}
				else
				{
					newObjective += solution.Instance.GetD(j);

					for (int i = solution.Instance.M - 1; i >= 0; i--)
					{
						var rowSum = solution.GetRowSum(i);
						rowSum += solution.Instance[i, j];
						if (rowSum > 0)
							newObjective += rowSum;
					}
				}

				if (newObjective > solution.Objective)
				{
					solution.SetY(j, !solution.GetY(j));
					for (int i = solution.Instance.M - 1; i >= 0; i--)
					{
						var rowSum = solution.GetRowSum(i);
						if (rowSum > 0)
							solution.SetX(i, true);
						else if (rowSum < 0)
							solution.SetX(i, false);
					}

					Debug.Assert(newObjective == solution.Objective);
				}
			}
		}

		public override bool FindsLocalMinimum
		{
			get { return false; }
		}

		public override bool CanWorsen
		{
			get { return false; }
		}

		public override bool IsDeterministicImprovement
		{
			get { return true; }
		}

		public override bool CanChangeSolution(AtomicOperator prevComponent, bool prevImprovedSolution)
		{
			if (prevComponent is FlipY && !prevImprovedSolution)
				return false;

			return true;
		}
	}
}