﻿using System;

namespace BbqpCmcs.AtomicOperators
{
	class Repair : AtomicOperator
	{
		private readonly int tournamentSize;
		private readonly int iterationsCount;

		public Repair(int tournamentSize, int iterationsCount)
		{
			this.tournamentSize = tournamentSize;
			this.iterationsCount = iterationsCount;
			Name = "Fix" + tournamentSize;
		}

		public override void Run(InternalBbqpSolution solution, Random random)
		{
			var instance = solution.Instance;

			int tournament = 0;
			int maxGain = -1;
			int bestRow = -1;
			int bestCol = -1;

			for (int iteration = 0; iteration < iterationsCount; iteration++)
			{
				int row = random.Next(instance.M);
				int col = random.Next(instance.N);

				int q = instance[row, col];
				if (q < 0)
				{
					if (solution.GetX(row) && solution.GetY(col))
					{
						tournament++;
						if (-q > maxGain)
						{
							maxGain = -q;
							bestRow = row;
							bestCol = col;
						}
					}
				}
				else if (q > 0)
				{
					if (!solution.GetX(row) || !solution.GetY(col))
					{
						tournament++;
						if (q > maxGain)
						{
							maxGain = q;
							bestRow = row;
							bestCol = col;
						}
					}
				}

				if (tournament >= tournamentSize)
				{
					if (instance[bestRow, bestCol] > 0)
					{
						solution.SetX(bestRow, true);
						solution.SetY(bestCol, true);
					}
					else
					{
						// There are two options
						long disableRow = -solution.GetRowSum(bestRow);
						long disableCol = -solution.GetColSum(bestCol);

						if (disableRow > disableCol)
							solution.SetX(bestRow, false);
						else
							solution.SetY(bestCol, false);
					}

					tournament = 0;
					maxGain = -1;
					bestRow = -1;
					bestCol = -1;

					return;
				}
			}
		}

		public override bool FindsLocalMinimum
		{
			get { return false; }
		}

		public override bool CanWorsen
		{
			get { return true; }
		}

		public override bool IsDeterministicImprovement
		{
			get { return false; }
		}
	}
}
