﻿using System;
using System.Diagnostics;

namespace BbqpCmcs.Types
{
	static class BbqpHelper
	{
		public const bool GenerationOfInstancesAllowed = false;

		public static long ImprovePortion(BbqpInstance instance, BbqpSolver portionSolver, int[] indices, bool[] x, bool[] y, long[] fullD, ref long fullC, int[][] qBuf, long[] cBuf)
		{
#if DEBUG
			for (int j = 0; j < instance.N; j++)
			{
				long fullDCheck = instance.GetD(j);
				for (int i1 = 0; i1 < instance.M; i1++)
				{
					if (x[i1])
						fullDCheck += instance[i1, j];
				}

				Debug.Assert(fullDCheck == fullD[j]);
			}
#endif

			for (int i = 0; i < indices.Length; i++)
			{
				var index = indices[i];
				if (!x[index])
					continue;

				fullC -= instance.GetC(index);
				for (int j = 0; j < instance.N; j++)
					fullD[j] -= instance[index, j];
			}

#if DEBUG
			for (int j = 0; j < instance.N; j++)
			{
				long fullDCheck = instance.GetD(j);
				for (int i1 = 0; i1 < instance.M; i1++)
				{
					if (x[i1] && Array.IndexOf(indices, i1) < 0)
						fullDCheck += instance[i1, j];
				}

				Debug.Assert(fullDCheck == fullD[j]);
			}
#endif

			//var c = new long[indices.Length];

			//var q = new int[indices.Length][];
			for (int i = 0; i < indices.Length; i++)
			{
				int index = indices[i];

				//q[i] = new int[instance.N];
				cBuf[i] = instance.GetC(index);

				for (int j = 0; j < instance.N; j++)
					qBuf[i][j] = instance[index, j];
			}


			var solution = (BbqpSolution)portionSolver.Solve(new BbqpInstance(null, null, qBuf, cBuf, fullD));
			long newObjective = fullC + solution.Objective;

			solution.CopyY(y);

			for (int i = 0; i < indices.Length; i++)
			{
				var index = indices[i];
				x[index] = solution.GetX(i);
				if (x[index])
				{
					fullC += instance.GetC(index);
					for (int j = 0; j < instance.N; j++)
						fullD[j] += instance[index, j];
				}
			}

#if DEBUG
			for (int j = 0; j < instance.N; j++)
			{
				long fullDCheck = instance.GetD(j);
				for (int i = 0; i < instance.M; i++)
				{
					if (x[i])
						fullDCheck += instance[i, j];
				}

				Debug.Assert(fullDCheck == fullD[j]);
			}
#endif

			return newObjective;
		}
	}
}
