﻿namespace BbqpCmcs.Types
{
	abstract class BbqpSolver : Solver<BbqpInstance>
	{
		protected BbqpSolver(string shortName) : base(shortName)
		{
		}

		protected BbqpSolver(string uniqueName, string shortName) : base(uniqueName, shortName)
		{
		}

		public abstract override Solution<BbqpInstance> Solve(BbqpInstance instance);

		protected bool ImproveX(BbqpInstance instance, bool[] x, bool[] y)
		{
			bool modified = false;
			int modif = 0;

#if DEBUG
			long oldObjective = new BbqpSolution(instance, x, y, TimeInterval.Zero, false).Objective;
#endif

			for (int i = 0; i < x.Length; i++)
			{
				long delta = instance.GetC(i);
				for (int j = 0; j < y.Length; j++)
					if (y[j])
						delta += instance[i, j];

				var newValue = delta > 0;
				modified |= delta != 0 && x[i] != newValue;
				if (delta != 0 && x[i] != newValue)
					modif++;
				x[i] = newValue;
			}

#if DEBUG
			var newObjective = new BbqpSolution(instance, x, y, TimeInterval.Zero, false).Objective;
			if (oldObjective > newObjective)
				throw new LabException("Improvement made the solution worse.");

			if (oldObjective == newObjective && modified)
				throw new LabException("No change in the objective is noticed.");
#endif

			//Helper.DebugMessage("ImproveX: {0}.", modif);

			return modified;
		}

		protected bool ImproveY(BbqpInstance instance, bool[] x, bool[] y)
		{
			bool modified = false;
			int modif = 0;

			for (int j = 0; j < y.Length; j++)
			{
				long delta = instance.GetD(j);
				for (int i = 0; i < x.Length; i++)
					if (x[i])
						delta += instance[i, j];

				var newValue = delta > 0;
				modified |= delta != 0 && y[j] != newValue;
				if (delta != 0 && y[j] != newValue)
					modif++;
				y[j] = newValue;
			}

			//Helper.DebugMessage("ImproveY: {0}.", modif);

			return modified;
		}

		public string Name
		{
			get { return ShortName; }
		}

		public int Count
		{
			get { return 1; }
		}
	}
}
